SystemOrganization addCategory: #VersionChangesCalculator!

Object subclass: #ChangeCriteria
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

ChangeCriteria subclass: #ChangeByCyclomaticComplexity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByCyclomaticComplexity methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass weightedMethodCount! !

ChangeCriteria subclass: #ChangeByLinesNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByLinesNum methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass numberOfLinesOfCode! !

ChangeCriteria subclass: #ChangeByMethodsNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByMethodsNum methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass numberOfMethods! !

!ChangeCriteria methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ 0! !

Object subclass: #ChangeInfo
	instanceVariableNames: 'changeNum versionNum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeInfo methodsFor: 'as yet unclassified' stamp: ''!
changeNum
	^ changeNum! !

!ChangeInfo methodsFor: 'as yet unclassified' stamp: ''!
changeNum: change 
	changeNum := change! !

!ChangeInfo methodsFor: 'as yet unclassified' stamp: ''!
versionNum
	^ versionNum! !

!ChangeInfo methodsFor: 'as yet unclassified' stamp: ''!
versionNum: version 
	versionNum := version! !

Object subclass: #MovingAverageCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateMovingAverage: values withPeriod: period 
	| movingAverages periodList periodCount totalPeriodValue |
	periodCount := 1.
	totalPeriodValue := 0.
	periodList := LinkedList new: 3.
	movingAverages := LinkedList new.
	values
		do: [:value | periodCount < period
				ifTrue: [periodCount := periodCount + 1.
					value isNil
						ifFalse: [totalPeriodValue := totalPeriodValue + value].
					periodList add: value]
				ifFalse: [periodList add: value.
					value isNil
						ifFalse: [totalPeriodValue := totalPeriodValue + value.
							totalPeriodValue := totalPeriodValue - periodList removeFirst].
					movingAverages add: totalPeriodValue / period]].
	^ movingAverages! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
createLineChart: values withMAValues: movAvgValues1 andMAValues2: movAvgValues2 
	| diag1 diag2 diag3 compDiag |
	diag1 := ESDiagramRenderer new lineDiagram y: #yourself;
				 models: values;
				 defaultColor: Color red;
				 yourself.
	diag2 := ESDiagramRenderer new lineDiagram y: #yourself;
				 defaultColor: Color green;
				 models: movAvgValues2;
				 yourself.
	diag3 := ESDiagramRenderer new lineDiagram y: #yourself;
				 defaultColor: Color blue;
				 models: movAvgValues1;
				 yourself.
	compDiag := ESDiagramRenderer new.
	compDiag compositeDiagram add: diag1;
		 add: diag2;
		 add: diag3;
		 preferredAxisMaxY: 45.
	^ compDiag! !

Object subclass: #VersionChangesCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassVersionChanges: classHistory withCriteria: changeCriteria numberOfVersions: numberOfVersions 
	| versionsChangesArray versionNum |
	versionsChangesArray := Array new: numberOfVersions.
	versionNum := 1.
	classHistory versions
		do: [:classVersion | 
			| previousClassVersion diffValue value1 value2 |
			versionNum := versionNum + 1.
			previousClassVersion := classVersion previousVersion.
			previousClassVersion isNil
				ifTrue: [diffValue := changeCriteria getNumber: classVersion]
				ifFalse: [value1 := changeCriteria getNumber: classVersion.
					value2 := changeCriteria getNumber: previousClassVersion.
					diffValue := (value1 - value2) abs].
			versionNum > 1
				ifTrue: [versionsChangesArray at: versionNum - 1 put: diffValue]].
	^ versionsChangesArray! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionChanges: hismoModel withCriteria: changeCriteria 
	| classesChangesDict classesHistories versionsNum |
	classesChangesDict := Dictionary new.
	classesHistories := hismoModel allClassHistories
				select: [:each | each last isStub not].
	versionsNum := hismoModel versions size.
	classesHistories
		do: [:classHistory | 
			| className changesArray |
			className := classHistory name.
			changesArray := self
						calculateClassVersionChanges: classHistory
						withCriteria: changeCriteria
						numberOfVersions: versionsNum.
			classesChangesDict at: className put: changesArray].
	^ classesChangesDict! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
createClassChangesLineChart: changesArray 
	| chart |
	chart := ESDiagramRenderer new.
	chart lineDiagram y: #changeNum;
		 yAxisLabel: 'Change Number';
		 identifier: #versionNum;
		 rotatedLabels: true;
		 regularAxis;
		 defaultColor: Color blue;
		
		preferredAxisMinY: (changesArray minValue: #changeNum);
		 models: changesArray.
	^ chart open! !

Object subclass: #VersionCriteriaCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassVersionCriteria: classHistory withCriteria: changeCriteria numberOfVersions: numberOfVersions 
	| versionsCriteriaArray versionNum |
	versionsCriteriaArray := Array new: numberOfVersions.
	versionNum := 1.
	classHistory versions
		do: [:classVersion | 
			| criteriaValue |
			criteriaValue := changeCriteria getNumber: classVersion.
			criteriaValue isNil
				ifTrue: [criteriaValue := 0].
			versionsCriteriaArray at: versionNum put: criteriaValue.
			versionNum := versionNum + 1].
	^ versionsCriteriaArray! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionCriteria: hismoModel withCriteria: changeCriteria 
	| classesCriteriaDict classesHistories versionsNum |
	classesCriteriaDict := Dictionary new.
	classesHistories := hismoModel allClassHistories
				select: [:each | each last isStub not].
	versionsNum := hismoModel versions size.
	classesHistories
		do: [:classHistory | 
			| className criteriaArray |
			className := classHistory name.
			criteriaArray := self
						calculateClassVersionCriteria: classHistory
						withCriteria: changeCriteria
						numberOfVersions: versionsNum.
			classesCriteriaDict at: className put: criteriaArray].
	^ classesCriteriaDict! !

Object subclass: #VolatilityCalculator
	instanceVariableNames: 'sum sum2 num'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateMean
	num > 0
		ifTrue: [^ sum / num].
	^ 0.0! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateStdDev
	^ self calculateVariance sqrt! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVariance
	| mean |
	mean := self calculateMean.
	num > 0
		ifTrue: [^ sum2 / num - (mean * mean)].
	^ 0.0! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVolatility: values 
	| revisionsNum volatility stdDev |
	sum := 0.
	sum2 := 0.
	num := 0.
	values
		do: [:value | 
			sum := sum + value.
			sum2 := sum2 + (value * value).
			num := num + 1.
			stdDev := self calculateStdDev].
	revisionsNum := values size.
	Transcript show: 'MEAN: ' , self calculateMean asString;
		 cr.
	Transcript show: 'STD DEV: ' , stdDev asString;
		 cr.
	Transcript show: 'VARIANCE: ' , self calculateVariance asString;
		 cr.
	^ stdDev * revisionsNum sqrt! !
